@@ -372,6 +372,7 @@ struct glk_window_struct
 
     window_t *parent; /* pair window which contains this one */
     rect_t bbox;
+    int yadj;
     void *data; /* one of the window_*_t structures */
 
     stream_t *str; /* the window stream. */
@@ -651,14 +652,16 @@ rect_t gli_compute_content_box();
 
 extern void gli_select(event_t *event, int polled);
 
+enum FILEFILTERS { FILTER_SAVE, FILTER_TEXT, FILTER_ALL };
+
 void wininit(int *argc, char **argv);
 void winopen(void);
 void wintitle(void);
 void winmore(void);
 void winrepaint(int x0, int y0, int x1, int y1);
 void winabort(const char *fmt, ...);
-void winopenfile(char *prompt, char *buf, int buflen, char *filter);
-void winsavefile(char *prompt, char *buf, int buflen, char *filter);
+void winopenfile(char *prompt, char *buf, int buflen, int filter);
+void winsavefile(char *prompt, char *buf, int buflen, int filter);
 void winexit(void);
 void winclipstore(glui32 *text, int len);
 
