@@ -5,14 +5,14 @@
 Product alan = {
   "Alan",
   "Adventure Language System",
-  "Alan 3.0alpha6",
-  "Alan 3.0alpha6 -- Adventure Language System (2008-09-28 22:04)",
-  "2008-09-28",
-  "22:04:40",
-  "thoni",
+  "Alan 3.0alpha8",
+  "Alan 3.0alpha8 -- Adventure Language System (2010-09-19 18:16)",
+  "2010-09-19",
+  "18:16:06",
+  "Thomas",
   "",
   "cygwin32",
-  {"3.0alpha6", 3, 0, 6, 1222632280, "alpha"}
+  {"3.0alpha8", 3, 0, 8, 1284912966, "alpha"}
 };
 
 static char *alanId =
